----------------------------------------------------------------------------
-- Session library.
----------------------------------------------------------------------------
-- $Id: session.lua,v 1.2 2007/01/31 13:11:37 zhaozg Exp $
----------------------------------------------------------------------------

local serialize = require"cgilua.serialize"

local assert, pairs, type, loadstring = assert, pairs, type, loadstring
local format, strfind, strrep, strsub = string.format, string.find, string.rep, string.sub
local ap = ap


-- Internal state variables.
local counter = 0
module ("cgilua.session")

----------------------------------------------------------------------------
-- Creates a new identifier.
----------------------------------------------------------------------------
local function new_id ()
	counter = counter + 1
	return format ("%08d", counter)
end

----------------------------------------------------------------------------
-- Checks identifier format.
----------------------------------------------------------------------------
local function check_id (id)
	return (strfind (id, strrep ("%d", 8)) ~= nil)
end

----------------------------------------------------------------------------
-- Deletes a session.
-- @param id Session identification.
----------------------------------------------------------------------------
function delete (id)
	assert (check_id (id))
	ap.storage(id,nil)
end

----------------------------------------------------------------------------
-- Creates a new session identifier.
-- @return Session identification.
----------------------------------------------------------------------------
function new ()
	local id = new_id ()
	while ap.storage(id) do
		id = new_id ()
	end
	return id
end

----------------------------------------------------------------------------
-- Loads data from a session.
-- @param id Session identification.
-- @return Table with session data.
----------------------------------------------------------------------------
function get (id)
	assert (check_id (id))

	local script = ap.storage(id)
	if (script) then
	  local f, err = loadstring (script)
 	  if not f then
		return nil, err
	  else
		return f()
	  end
	else
	  return nil, "Not exist session "..id
	end
end

----------------------------------------------------------------------------
-- Saves data to a session.
-- @param id Session identification.
-- @param data Table with session data to be saved.
----------------------------------------------------------------------------
function set (id, data)
	assert (check_id (id))

	local content = "";
	content = content.."return "
	serialize(data, function (s) content = content..s end)
	ap.storage(id, content);
end


